@@ -245,7 +245,7 @@ InflateVarTag(mat_t *mat, matvar_t *matvar, void *buf)
     int    err;
     size_t bytesread = 0;
 
-    if (buf == NULL)
+    if ( buf == NULL )
         return 0;
 
     if ( !matvar->internal->z->avail_in ) {
@@ -296,7 +296,8 @@ InflateArrayFlags(mat_t *mat, matvar_t *matvar, void *buf)
     int    err;
     size_t bytesread = 0;
 
-    if (buf == NULL) return 0;
+    if ( buf == NULL )
+        return 0;
 
     if ( !matvar->internal->z->avail_in ) {
         matvar->internal->z->avail_in = 1;
