@@ -1,5 +1,5 @@
 /** @file mat.c
- * Matlab MAT version 5 file functions
+ * Matlab MAT file functions
  * @ingroup MAT
  */
 /*
@@ -156,6 +156,7 @@ ComplexMalloc(size_t nbytes)
             complex_data = NULL;
         }
     }
+
     return complex_data;
 }
 
@@ -284,7 +285,9 @@ Mat_Open(const char *matname,int mode)
     mat->version       = 0;
     mat->byteswap      = 0;
     mat->num_datasets  = 0;
+#if defined(MAT73) && MAT73
     mat->refs_id       = -1;
+#endif
     mat->dir           = NULL;
 
     bytesread += fread(mat->header,1,116,fp);
@@ -296,9 +299,9 @@ Mat_Open(const char *matname,int mode)
     if ( 128 == bytesread ) {
         /* v5 and v7.3 files have at least 128 byte header */
         mat->byteswap = -1;
-        if (tmp == 0x4d49)
+        if ( tmp == 0x4d49 )
             mat->byteswap = 0;
-        else if (tmp == 0x494d) {
+        else if ( tmp == 0x494d ) {
             mat->byteswap = 1;
             Mat_int16Swap(&tmp2);
         }
@@ -336,7 +339,9 @@ Mat_Open(const char *matname,int mode)
         mat->mode          = mode;
         mat->bof           = 0;
         mat->next_index    = 0;
+#if defined(MAT73) && MAT73
         mat->refs_id       = -1;
+#endif
 
         Mat_Rewind(mat);
         var = Mat_VarReadNextInfo4(mat);
@@ -391,17 +396,20 @@ Mat_Open(const char *matname,int mode)
  * Closes the given Matlab MAT file and frees any memory with it.
  * @ingroup MAT
  * @param mat Pointer to the MAT file
- * @retval 0
+ * @retval 0 on success
  */
 int
 Mat_Close( mat_t *mat )
 {
+    int err = 0;
+
     if ( NULL != mat ) {
 #if defined(MAT73) && MAT73
         if ( mat->version == 0x0200 ) {
             if ( mat->refs_id > -1 )
                 H5Gclose(mat->refs_id);
-            H5Fclose(*(hid_t*)mat->fp);
+            if ( 0 > H5Fclose(*(hid_t*)mat->fp) )
+                err = 1;
             free(mat->fp);
             mat->fp = NULL;
         }
@@ -424,7 +432,8 @@ Mat_Close( mat_t *mat )
         }
         free(mat);
     }
-    return 0;
+
+    return err;
 }
 
 /** @brief Gets the filename for the given MAT file
@@ -486,12 +495,12 @@ Mat_GetDir(mat_t *mat, size_t *n)
         if ( mat->version == MAT_FT_MAT73 ) {
             size_t i = 0;
             size_t fpos = mat->next_index;
-            if ( mat->num_datasets == 0) {
+            if ( mat->num_datasets == 0 ) {
                 *n = 0;
                 return dir;
             }
             mat->dir = calloc(mat->num_datasets, sizeof(char*));
-            if ( NULL == mat->dir) {
+            if ( NULL == mat->dir ) {
                 *n = 0;
                 Mat_Critical("Couldn't allocate memory for the directory");
                 return dir;
@@ -541,7 +550,7 @@ Mat_GetDir(mat_t *mat, size_t *n)
                         }
                     }
                     Mat_VarFree(matvar);
-                } else if (!feof((FILE *)mat->fp)) {
+                } else if ( !feof((FILE *)mat->fp) ) {
                     Mat_Critical("An error occurred in reading the MAT file");
                     break;
                 }
@@ -594,7 +603,7 @@ Mat_Rewind( mat_t *mat )
 size_t
 Mat_SizeOfClass(int class_type)
 {
-    switch (class_type) {
+    switch ( class_type ) {
         case MAT_C_DOUBLE:
             return sizeof(double);
         case MAT_C_SINGLE:
@@ -663,12 +672,12 @@ Mat_VarCalloc(void)
             free(matvar);
             matvar = NULL;
         } else {
+#if defined(MAT73) && MAT73
             matvar->internal->hdf5_name  = NULL;
             matvar->internal->hdf5_ref   =  0;
             matvar->internal->id         = -1;
-            matvar->internal->fpos       = 0;
+#endif
             matvar->internal->datapos    = 0;
-            matvar->internal->fp         = NULL;
             matvar->internal->num_fields = 0;
             matvar->internal->fieldnames = NULL;
 #if defined(HAVE_ZLIB)
@@ -744,7 +753,8 @@ Mat_VarCreate(const char *name,enum matio_classes class_type,
     size_t i, nmemb = 1, data_size;
     matvar_t *matvar = NULL;
 
-    if (dims == NULL) return NULL;
+    if ( dims == NULL )
+        return NULL;
 
     matvar = Mat_VarCalloc();
     if ( NULL == matvar )
@@ -929,20 +939,20 @@ mat_copy(const char* src, const char* dst)
     FILE* out;
 
     in = fopen(src, "rb");
-    if (in == NULL) {
+    if ( in == NULL ) {
         Mat_Critical("Cannot open file \"%s\" for reading.", src);
         return -1;
     }
 
     out = fopen(dst, "wb");
-    if (out == NULL) {
+    if ( out == NULL ) {
         fclose(in);
         Mat_Critical("Cannot open file \"%s\" for writing.", dst);
         return -1;
     }
 
-    while ((len = fread(buf, sizeof(char), BUFSIZ, in)) > 0) {
-        if (len != fwrite(buf, sizeof(char), len, out)) {
+    while ( (len = fread(buf, sizeof(char), BUFSIZ, in)) > 0 ) {
+        if ( len != fwrite(buf, sizeof(char), len, out) ) {
             fclose(in);
             fclose(out);
             Mat_Critical("Error writing to file \"%s\".", dst);
@@ -1116,98 +1126,100 @@ Mat_VarDuplicate(const matvar_t *in, int opt)
     out->mem_conserve = in->mem_conserve;
     out->compression  = in->compression;
 
-    out->name = NULL;
-    out->dims = NULL;
-    out->data = NULL;
-
-    if ( NULL != in->internal->hdf5_name )
-        out->internal->hdf5_name = strdup(in->internal->hdf5_name);
-
-    out->internal->hdf5_ref = in->internal->hdf5_ref;
-    out->internal->id       = in->internal->id;
-    out->internal->fpos     = in->internal->fpos;
-    out->internal->datapos  = in->internal->datapos;
-#if defined(HAVE_ZLIB)
-    out->internal->z        = NULL;
-    out->internal->data     = NULL;
-#endif
-    out->internal->num_fields = in->internal->num_fields;
-    if ( NULL != in->internal->fieldnames && in->internal->num_fields > 0 ) {
-        out->internal->fieldnames = (char**)calloc(in->internal->num_fields,
-                                           sizeof(*in->internal->fieldnames));
-        for ( i = 0; i < in->internal->num_fields; i++ ) {
-            if ( NULL != in->internal->fieldnames[i] )
-                out->internal->fieldnames[i] =
-                    strdup(in->internal->fieldnames[i]);
-        }
-    }
-
-    if (in->name != NULL && (NULL != (out->name = (char*)malloc(strlen(in->name)+1))))
+    if ( in->name != NULL && (NULL != (out->name = (char*)malloc(strlen(in->name)+1))) )
         memcpy(out->name,in->name,strlen(in->name)+1);
 
     out->dims = (size_t*)malloc(in->rank*sizeof(*out->dims));
     if ( out->dims != NULL )
         memcpy(out->dims,in->dims,in->rank*sizeof(*out->dims));
 
+    if ( NULL != in->internal ) {
+#if defined(MAT73) && MAT73
+        if ( NULL != in->internal->hdf5_name )
+            out->internal->hdf5_name = strdup(in->internal->hdf5_name);
+
+        out->internal->hdf5_ref = in->internal->hdf5_ref;
+        out->internal->id       = in->internal->id;
+#endif
+        out->internal->datapos  = in->internal->datapos;
 #if defined(HAVE_ZLIB)
-    if ( (in->internal->z != NULL) && (NULL != (out->internal->z = (z_streamp)malloc(sizeof(z_stream)))) )
-        inflateCopy(out->internal->z,in->internal->z);
-    if ( in->internal->data != NULL ) {
-        if ( in->class_type == MAT_C_SPARSE ) {
-            out->internal->data = malloc(sizeof(mat_sparse_t));
-            if ( out->internal->data != NULL ) {
-                mat_sparse_t *out_sparse = (mat_sparse_t*)out->internal->data;
-                mat_sparse_t *in_sparse  = (mat_sparse_t*)in->internal->data;
-                out_sparse->nzmax = in_sparse->nzmax;
-                out_sparse->nir = in_sparse->nir;
-                out_sparse->ir = (int*)malloc(in_sparse->nir*sizeof(int));
-                if ( out_sparse->ir != NULL )
-                    memcpy(out_sparse->ir, in_sparse->ir, in_sparse->nir*sizeof(int));
-                out_sparse->njc = in_sparse->njc;
-                out_sparse->jc = (int*)malloc(in_sparse->njc*sizeof(int));
-                if ( out_sparse->jc != NULL )
-                    memcpy(out_sparse->jc, in_sparse->jc, in_sparse->njc*sizeof(int));
-                out_sparse->ndata = in_sparse->ndata;
-                if ( out->isComplex && NULL != in_sparse->data ) {
-                    out_sparse->data = malloc(sizeof(mat_complex_split_t));
-                    if ( out_sparse->data != NULL ) {
-                        mat_complex_split_t *out_data = (mat_complex_split_t*)out_sparse->data;
-                        mat_complex_split_t *in_data  = (mat_complex_split_t*)in_sparse->data;
-                        out_data->Re = malloc(
-                            in_sparse->ndata*Mat_SizeOf(in->data_type));
-                        if ( NULL != out_data->Re )
-                            memcpy(out_data->Re,in_data->Re,
+        out->internal->z        = NULL;
+        out->internal->data     = NULL;
+#endif
+        out->internal->num_fields = in->internal->num_fields;
+        if ( NULL != in->internal->fieldnames && in->internal->num_fields > 0 ) {
+            out->internal->fieldnames = (char**)calloc(in->internal->num_fields,
+                                               sizeof(*in->internal->fieldnames));
+            for ( i = 0; i < in->internal->num_fields; i++ ) {
+                if ( NULL != in->internal->fieldnames[i] )
+                    out->internal->fieldnames[i] =
+                        strdup(in->internal->fieldnames[i]);
+            }
+        }
+
+#if defined(HAVE_ZLIB)
+        if ( (in->internal->z != NULL) && (NULL != (out->internal->z = (z_streamp)malloc(sizeof(z_stream)))) )
+            inflateCopy(out->internal->z,in->internal->z);
+        if ( in->internal->data != NULL ) {
+            if ( in->class_type == MAT_C_SPARSE ) {
+                out->internal->data = malloc(sizeof(mat_sparse_t));
+                if ( out->internal->data != NULL ) {
+                    mat_sparse_t *out_sparse = (mat_sparse_t*)out->internal->data;
+                    mat_sparse_t *in_sparse  = (mat_sparse_t*)in->internal->data;
+                    out_sparse->nzmax = in_sparse->nzmax;
+                    out_sparse->nir = in_sparse->nir;
+                    out_sparse->ir = (int*)malloc(in_sparse->nir*sizeof(int));
+                    if ( out_sparse->ir != NULL )
+                        memcpy(out_sparse->ir, in_sparse->ir, in_sparse->nir*sizeof(int));
+                    out_sparse->njc = in_sparse->njc;
+                    out_sparse->jc = (int*)malloc(in_sparse->njc*sizeof(int));
+                    if ( out_sparse->jc != NULL )
+                        memcpy(out_sparse->jc, in_sparse->jc, in_sparse->njc*sizeof(int));
+                    out_sparse->ndata = in_sparse->ndata;
+                    if ( out->isComplex && NULL != in_sparse->data ) {
+                        out_sparse->data = malloc(sizeof(mat_complex_split_t));
+                        if ( out_sparse->data != NULL ) {
+                            mat_complex_split_t *out_data = (mat_complex_split_t*)out_sparse->data;
+                            mat_complex_split_t *in_data  = (mat_complex_split_t*)in_sparse->data;
+                            out_data->Re = malloc(
                                 in_sparse->ndata*Mat_SizeOf(in->data_type));
-                        out_data->Im = malloc(
-                            in_sparse->ndata*Mat_SizeOf(in->data_type));
-                        if ( NULL != out_data->Im )
-                            memcpy(out_data->Im,in_data->Im,
+                            if ( NULL != out_data->Re )
+                                memcpy(out_data->Re,in_data->Re,
+                                    in_sparse->ndata*Mat_SizeOf(in->data_type));
+                            out_data->Im = malloc(
+                                in_sparse->ndata*Mat_SizeOf(in->data_type));
+                            if ( NULL != out_data->Im )
+                                memcpy(out_data->Im,in_data->Im,
+                                    in_sparse->ndata*Mat_SizeOf(in->data_type));
+                        }
+                    } else if ( in_sparse->data != NULL ) {
+                        out_sparse->data = malloc(in_sparse->ndata*Mat_SizeOf(in->data_type));
+                        if ( NULL != out_sparse->data )
+                            memcpy(out_sparse->data, in_sparse->data,
                                 in_sparse->ndata*Mat_SizeOf(in->data_type));
                     }
-                } else if ( in_sparse->data != NULL ) {
-                    out_sparse->data = malloc(in_sparse->ndata*Mat_SizeOf(in->data_type));
-                    if ( NULL != out_sparse->data )
-                        memcpy(out_sparse->data, in_sparse->data,
-                            in_sparse->ndata*Mat_SizeOf(in->data_type));
                 }
+            } else if ( out->isComplex ) {
+                out->internal->data = malloc(sizeof(mat_complex_split_t));
+                if ( out->internal->data != NULL ) {
+                    mat_complex_split_t *out_data = (mat_complex_split_t*)out->internal->data;
+                    mat_complex_split_t *in_data  = (mat_complex_split_t*)in->internal->data;
+                    out_data->Re = malloc(out->nbytes);
+                    if ( NULL != out_data->Re )
+                        memcpy(out_data->Re,in_data->Re,out->nbytes);
+                    out_data->Im = malloc(out->nbytes);
+                    if ( NULL != out_data->Im )
+                        memcpy(out_data->Im,in_data->Im,out->nbytes);
+                }
+            } else if ( NULL != (out->internal->data = malloc(in->nbytes)) ) {
+                memcpy(out->internal->data, in->internal->data, in->nbytes);
             }
-        } else if ( out->isComplex ) {
-            out->internal->data = malloc(sizeof(mat_complex_split_t));
-            if ( out->internal->data != NULL ) {
-                mat_complex_split_t *out_data = (mat_complex_split_t*)out->internal->data;
-                mat_complex_split_t *in_data  = (mat_complex_split_t*)in->internal->data;
-                out_data->Re = malloc(out->nbytes);
-                if ( NULL != out_data->Re )
-                    memcpy(out_data->Re,in_data->Re,out->nbytes);
-                out_data->Im = malloc(out->nbytes);
-                if ( NULL != out_data->Im )
-                    memcpy(out_data->Im,in_data->Im,out->nbytes);
-            }
-        } else if ( NULL != (out->internal->data = malloc(in->nbytes)) ) {
-            memcpy(out->internal->data, in->internal->data, in->nbytes);
         }
-    }
 #endif
+    } else {
+        free(out->internal);
+        out->internal = NULL;
+    }
 
     if ( !opt ) {
         out->data = in->data;
@@ -1285,6 +1297,7 @@ Mat_VarDuplicate(const matvar_t *in, int opt)
                 memcpy(out->data,in->data,in->nbytes);
         }
     }
+
     return out;
 }
 
@@ -1307,7 +1320,7 @@ Mat_VarFree(matvar_t *matvar)
             nmemb *= matvar->dims[i];
         free(matvar->dims);
     }
-    if ( NULL != matvar->data) {
+    if ( NULL != matvar->data ) {
         switch (matvar->class_type ) {
             case MAT_C_STRUCT:
                 if ( !matvar->mem_conserve ) {
@@ -1458,10 +1471,6 @@ Mat_VarFree(matvar_t *matvar)
     }
     if ( NULL != matvar->name )
         free(matvar->name);
-    /* FIXME: Why does this cause a SEGV? */
-#if 0
-    memset(matvar,0,sizeof(matvar_t));
-#endif
     free(matvar);
 }
 
@@ -1637,15 +1646,16 @@ size_t
 Mat_VarGetSize(matvar_t *matvar)
 {
     int i;
-    size_t bytes = 0;
-    size_t overhead = 0;
+    size_t bytes = 0, overhead = 0, ptr = 0;
 
 #if defined(_WIN64) || (defined(__SIZEOF_POINTER__) && (__SIZEOF_POINTER__ == 8)) || (defined(SIZEOF_VOID_P) && (SIZEOF_VOID_P == 8))
     /* 112 bytes cell/struct overhead for 64-bit system */
     overhead = 112;
+    ptr = 8;
 #elif defined(_WIN32) || (defined(__SIZEOF_POINTER__) && (__SIZEOF_POINTER__ == 4)) || (defined(SIZEOF_VOID_P) && (SIZEOF_VOID_P == 4))
     /* 60 bytes cell/struct overhead for 32-bit system */
     overhead = 60;
+    ptr = 4;
 #endif
 
     if ( matvar->class_type == MAT_C_STRUCT ) {
@@ -1657,9 +1667,14 @@ Mat_VarGetSize(matvar_t *matvar)
             matvar_t **fields = (matvar_t**)matvar->data;
             if ( NULL != fields ) {
                 bytes = nmemb*nfields*overhead;
-                for ( i = 0; i < nmemb*nfields; i++ )
-                    if ( NULL != fields[i] )
-                        bytes += Mat_VarGetSize(fields[i]);
+                for ( i = 0; i < nmemb*nfields; i++ ) {
+                    if ( NULL != fields[i] ) {
+                        if ( MAT_C_EMPTY != fields[i]->class_type )
+                            bytes += Mat_VarGetSize(fields[i]);
+                        else
+                            bytes += ptr - overhead;
+                    }
+                }
             }
         }
         bytes += 64 /* max field name length */ *nfields;
@@ -1668,9 +1683,14 @@ Mat_VarGetSize(matvar_t *matvar)
         if ( NULL != cells ) {
             int ncells = matvar->nbytes / matvar->data_size;
             bytes = ncells*overhead;
-            for ( i = 0; i < ncells; i++ )
-                if ( NULL != cells[i] )
-                    bytes += Mat_VarGetSize(cells[i]);
+            for ( i = 0; i < ncells; i++ ) {
+                if ( NULL != cells[i] ) {
+                    if ( MAT_C_EMPTY != cells[i]->class_type )
+                        bytes += Mat_VarGetSize(cells[i]);
+                    else
+                        bytes += ptr - overhead;
+                }
+            }
         }
     } else if ( matvar->class_type == MAT_C_SPARSE ) {
         mat_sparse_t *sparse = (mat_sparse_t*)matvar->data;
@@ -1696,6 +1716,7 @@ Mat_VarGetSize(matvar_t *matvar)
         if ( matvar->isComplex )
             bytes *= 2;
     }
+
     return bytes;
 }
 
@@ -1718,16 +1739,6 @@ Mat_VarPrint( matvar_t *matvar, int printdata )
        "16-bit, unsigned integer array","32-bit, signed integer array",
        "32-bit, unsigned integer array","64-bit, signed integer array",
        "64-bit, unsigned integer array"};
-    const char *data_type_desc[23] = {"Unknown","8-bit, signed integer",
-       "8-bit, unsigned integer","16-bit, signed integer",
-       "16-bit, unsigned integer","32-bit, signed integer",
-       "32-bit, unsigned integer","IEEE 754 single-precision","RESERVED",
-       "IEEE 754 double-precision","RESERVED","RESERVED",
-       "64-bit, signed integer","64-bit, unsigned integer", "Matlab Array",
-       "Compressed Data","Unicode UTF-8 Encoded Character Data",
-       "Unicode UTF-16 Encoded Character Data",
-       "Unicode UTF-32 Encoded Character Data","","String","Cell Array",
-       "Structure"};
 
     if ( matvar == NULL )
         return;
@@ -1749,8 +1760,19 @@ Mat_VarPrint( matvar_t *matvar, int printdata )
     else if ( matvar->isLogical )
         printf(" (logical)");
     printf("\n");
-    if ( matvar->data_type )
+    if ( matvar->data_type ) {
+        const char *data_type_desc[23] = {"Unknown","8-bit, signed integer",
+           "8-bit, unsigned integer","16-bit, signed integer",
+           "16-bit, unsigned integer","32-bit, signed integer",
+           "32-bit, unsigned integer","IEEE 754 single-precision","RESERVED",
+           "IEEE 754 double-precision","RESERVED","RESERVED",
+           "64-bit, signed integer","64-bit, unsigned integer", "Matlab Array",
+           "Compressed Data","Unicode UTF-8 Encoded Character Data",
+           "Unicode UTF-16 Encoded Character Data",
+           "Unicode UTF-32 Encoded Character Data","","String","Cell Array",
+           "Structure"};
         printf(" Data Type: %s\n", data_type_desc[matvar->data_type]);
+    }
 
     if ( MAT_C_STRUCT == matvar->class_type ) {
         matvar_t **fields = (matvar_t **)matvar->data;
@@ -2126,7 +2148,7 @@ Mat_VarReadInfo( mat_t *mat, const char *name )
                         Mat_VarFree(matvar);
                         matvar = NULL;
                     }
-                } else if (!feof((FILE *)mat->fp)) {
+                } else if ( !feof((FILE *)mat->fp) ) {
                     Mat_Critical("An error occurred in reading the MAT file");
                     break;
                 }
@@ -2136,6 +2158,7 @@ Mat_VarReadInfo( mat_t *mat, const char *name )
             Mat_Critical("Couldn't determine file position");
         }
     }
+
     return matvar;
 }
 
@@ -2174,6 +2197,7 @@ Mat_VarRead( mat_t *mat, const char *name )
             ReadData(mat,matvar);
         mat->next_index = fpos;
     }
+
     return matvar;
 }
 
@@ -2204,7 +2228,7 @@ Mat_VarReadNext( mat_t *mat )
     matvar = Mat_VarReadNextInfo(mat);
     if ( matvar ) {
         ReadData(mat,matvar);
-    } else if (mat->version != MAT_FT_MAT73 ) {
+    } else if ( mat->version != MAT_FT_MAT73 ) {
         (void)fseek((FILE*)mat->fp,fpos,SEEK_SET);
     }
 
@@ -2257,7 +2281,8 @@ Mat_VarWriteData(mat_t *mat,matvar_t *matvar,void *data,
     if ( mat == NULL || matvar == NULL )
         return -1;
 
-    (void)fseek((FILE*)mat->fp,matvar->internal->datapos+8,SEEK_SET);
+    if ( NULL != matvar->internal )
+        (void)fseek((FILE*)mat->fp,matvar->internal->datapos+8,SEEK_SET);
 
     if ( data == NULL ) {
         err = -1;
@@ -2329,13 +2354,13 @@ Mat_VarWrite(mat_t *mat,matvar_t *matvar,enum matio_compression compress)
     if ( NULL == mat || NULL == matvar )
         return -1;
 
-    if ( NULL == mat->dir) {
+    if ( NULL == mat->dir ) {
         size_t n = 0;
         (void)Mat_GetDir(mat, &n);
     }
 
     {
-        /* Error if MAT variable already exists in MAT file*/
+        /* Error if MAT variable already exists in MAT file */
         size_t i;
         for ( i = 0; i < mat->num_datasets; i++ ) {
             if ( NULL != mat->dir[i] &&
@@ -2360,6 +2385,81 @@ Mat_VarWrite(mat_t *mat,matvar_t *matvar,enum matio_compression compress)
         err = 2;
 
     if ( err == 0 ) {
+        /* Update directory */
+        char **dir;
+        if ( NULL == mat->dir ) {
+            dir = malloc(sizeof(char*));
+        } else {
+            dir = realloc(mat->dir,
+            (mat->num_datasets + 1)*(sizeof(char*)));
+        }
+        if ( NULL != dir ) {
+            mat->dir = dir;
+            if ( NULL != matvar->name ) {
+                mat->dir[mat->num_datasets++] =
+                    strdup_printf("%s", matvar->name);
+            } else {
+                mat->dir[mat->num_datasets++] = NULL;
+            }
+        } else {
+            err = 3;
+            Mat_Critical("Couldn't allocate memory for the directory");
+        }
+    }
+
+    return err;
+}
+
+/** @brief Writes/appends the given MAT variable to a version 7.3 MAT file
+ *
+ * Writes the numeric data of the MAT variable stored in matvar to the given
+ * MAT file. The variable will be written to the end of the file if it does
+ * not yet exist or appended to the exsiting variable.
+ * @ingroup MAT
+ * @param mat MAT file to write to
+ * @param matvar MAT variable information to write
+ * @param compress Whether or not to compress the data
+ *        (Only valid for version 7.3 MAT files and variables with numeric data)
+ * @param dim dimension to append data
+ *        (Only valid for version 7.3 MAT files and variables with numeric data)
+ * @retval 0 on success
+ */
+int
+Mat_VarWriteAppend(mat_t *mat,matvar_t *matvar,enum matio_compression compress,int dim)
+{
+    int err, append = 0;
+
+    if ( NULL == mat || NULL == matvar )
+        return -1;
+
+    if ( NULL == mat->dir ) {
+        size_t n = 0;
+        (void)Mat_GetDir(mat, &n);
+    }
+
+    {
+        /* Check if MAT variable already exists in MAT file */
+        size_t i;
+        for ( i = 0; i < mat->num_datasets; i++ ) {
+            if ( NULL != mat->dir[i] &&
+                0 == strcmp(mat->dir[i], matvar->name) ) {
+                append = 1;
+                break;
+            }
+        }
+    }
+
+    if ( mat->version == MAT_FT_MAT73 )
+#if defined(MAT73) && MAT73
+        err = Mat_VarWriteAppend73(mat,matvar,compress,dim);
+#else
+        err = 1;
+#endif
+    else
+        err = 2;
+
+    if ( err == 0 && 0 == append ) {
+        /* Update directory */
         char **dir;
         if ( NULL == mat->dir ) {
             dir = malloc(sizeof(char*));
